@@ -3,7 +3,7 @@ use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = '1.030';
+our $VERSION = '1.031';
 
 =head1 NAME
 
@@ -675,13 +675,30 @@ sub pack_manifest_hash {
     my $par_file = $self->{par_file};
     my (@modules, @data, @exclude);
 
+    # Search for scannable code in all -I'd paths
+    # NOTE: We need this early, because Module::ScanDeps::_find_in_inc 
+    # and others refer to @IncludeLibs.
+    push @Module::ScanDeps::IncludeLibs, @{$opt->{I}} if $opt->{I};
+    
     foreach my $name (@{ $opt->{M} || [] }) {
-        # for "-M Foo::Bar::" also add everything below .../Foo/Bar
-        if ($name =~ s/^([\w:]+)::$/$1/) {
+        if ($name =~ s/^([\w:]+)::(\*{0,2})$/$1/) {
+            my $ns = length $2;        # namespace depth indicator
+
+            if ($ns == 0) {
+                # "-MFoo::" shorthand for "-MFoo -MFoo::**"
+                $self->_name2moddata($name, \@modules, \@data);
+                $ns = 2;
+            }
+
             (my $mod = $name) =~ s/::/\//g;
-            Module::ScanDeps::add_preload_rule("$mod.pm", 'sub');
+            my @mods_in_ns = $ns == 1
+                ? Module::ScanDeps::_glob_in_inc_1($mod, 1)
+                : Module::ScanDeps::_glob_in_inc($mod, 1);
+            $self->_name2moddata($_, \@modules, \@data) foreach @mods_in_ns;
+        } 
+        else {
+            $self->_name2moddata($name, \@modules, \@data);
         }
-        $self->_name2moddata($name, \@modules, \@data);
     }
 
     if ($opt->{u}) {
@@ -708,9 +725,7 @@ sub pack_manifest_hash {
     unshift(@INC, @{ $opt->{I} || [] });
     unshift(@SharedLibs, map $self->_find_shlib($_), @{ $opt->{l} || [] });
 
-    my $inc_find = $self->_obj_function($fe, '_find_in_inc');
-
-    my %skip = map { $_, 1 } map &$inc_find($_), @exclude;
+    my %skip = map { Module::ScanDeps::_find_in_inc($_), 1 } @exclude;
     if ($^O eq 'MSWin32') {
         %skip = (%skip, map { s{\\}{/}g; lc($_), 1 } @SharedLibs);
     }
@@ -724,7 +739,7 @@ sub pack_manifest_hash {
     # Apply %Preload to the -M'd modules and add them to the list of
     # files to scan
     foreach my $module (@modules) {
-        my $file = &$inc_find($module)
+        my $file = Module::ScanDeps::_find_in_inc($module)
           or $self->_die("Cannot find module $module (specified with -M)\n");
         push @files, $file;
         
@@ -740,9 +755,6 @@ sub pack_manifest_hash {
     }
     push @files, @$input;
 
-    # Search for scannable code in all -I'd paths
-    push @Module::ScanDeps::IncludeLibs, @{$opt->{I}} if $opt->{I};
-    
     if ($opt->{x} && defined $opt->{xargs}) {
         require Text::ParseWords;
         $opt->{x} = [ Text::ParseWords::shellwords($opt->{xargs}) ];
@@ -767,7 +779,7 @@ sub pack_manifest_hash {
         ),
     );
 
-    %skip = map { $_, 1 } map &$inc_find($_), @exclude;
+    %skip = map { Module::ScanDeps::_find_in_inc($_), 1 } @exclude;
     %skip = (%skip, map { $_, 1 } @SharedLibs);
 
     $add_deps->(
